/***************************************************************************
                          devices.h  -  description
                             -------------------
    begin                : Fri May 30 2003
    copyright            : (C) 2003 by Piotr Mierzwiski
    email                : peterm@go2.pl

    copyright            : See COPYING file that comes with this project
 $Id: devicesmenu.h 636 2009-06-14 14:52:19Z piotr $
 ***************************************************************************/

#ifndef DEVICES_H
#define DEVICES_H

#include <qdialog.h>
#include <qwidget.h>
#include <qprocess.h>
#include <qpopupmenu.h>

/**
 * Author:
 * Piotr Mierzwiski
 **/
/** @short Klasa obsuguje menu z list urzdze masowych.
 W klasie inicjowane jest menu z list urzdze oraz stan ich zamontowania.
 Wykorzystywane s do tego dwa pliki: /etc/fstab, /etc/mtab (stan zamontowania).
 Po wybraniu urzdzenia przez uytkownika uruchamiana jest odpowiednia akcja.
 Mianowicie: jeli urzdzenie jest zamontowane, wtedy nastpuje jego
 odmontowanie, jeeli nie jest zamontowane, wtedy podejmowana jest prba
 zamontowania i trzecim przypadku, kiedy urzdzenie jest zamontowane a
 uytkownik je wybierze, wtedy wysyany jest sygna @em signalPathChanged() ,
 ktry wymusza zmian biecej cieki.
*/
class DevicesMenu: public QPopupMenu
{
	Q_OBJECT
public:
	/** Konstruktor klasy.
	 * Utworzenie obiektu QProcess
	@param pParent - wskanik na rodzica.
	 */
	DevicesMenu( QWidget *pParent );

	/** Destruktor.\n
	 Usuwane s tu obiekty uywane w klasie.
	 */
	~DevicesMenu();

	/** Rodzaj operacji jaki mona wykona na biecym elemencie menu.
	 */
	enum Operation { OPENDIR=0, MOUNT, UNMOUNT };

	/** Powoduje pokazanie menu z list dostpnych urzdze.\n
	 Pokazywane jest ono na pozycji (3,3) wzgldem obiektu rodzica. Po wybraniu
	 opcji uruchamiana jest odpowiednia akcja. Jeli urzdzenie jest zamontowane,
	 wtedy nastpuje jego odmontowanie, jeeli nie jest zamontowane, wtedy
	 podejmowana jest prba zamontowania i trzecim przypadku kiedy urzdzenie jest
	 zamontowane a uytkownik je wybierze wtedy wysyany jest tylko sygna
	 @em signalPathChanged(), ktry wymusza zmian biecej cieki.
	 */
	void showMenu();

private:
	QValueList<bool> m_vlMountedTab;
	Operation m_eCurrentOperation;

	QProcess *m_pProcess;
	QDialog  *m_pInfoDlg;
	QString   m_sMountPath;

	bool m_bActionIsFinished;

	/**
	Wczytywane s tutaj dwa pliki: /etc/fstab, /etc/mtab. Nastpnie inicjowane
	jest menu list dostpnych urzdze oraz stanem ich zamot
	 @return status wczytania fstab i mtab
	 */
	bool init();

	/** Uruchamia (asynchronicznie) akcj na, ktr wskazuje skadowa zawierajca
	 polecenie.\n
	 Nastpnie pokazywy jest dialog z prob o cierpliwo i guzikiem
	 umoliwiajcym przerwanie operacji.
	 */
	void action();

	/** Powoduje uruchamienie podanego polecenia z podanymi argumentami.\n
	 @param op - operacja do wykonania, patrz: @see Operation,
	 @param args - argumenty dla polecenia.
	 */
	void runProcess( Operation op, const QString & args );

private slots:
	/** Zawiera obsug zakoczenia wykonywania polecenia.\n
	 Jeli podczas operacji wystpi bd, wtedy tutaj pokazywany jest stosowny
	 komunikat.
	 */
	void slotProcessExited();

	/** Powoduje przerwanie biecej operacji.
	 */
	void slotBreakProcess();

	/** Uruchamia proces odmontowywania biecego urzdzenia.
	 */
	void slotUnmount();

	/** Uruchamia proces zamontowania biecego urzdzenia.
	 */
	void slotMount();

signals:
	/** Sygna wymuszajcy zmian cieki w biecym panelu na podan.\n
	 @param path - nowa cieka do katalogu.
	 */
	void signalPathChanged( const QString & path );

};

#endif
