/***************************************************************************
                          fileview.h  -  description
                             -------------------
    begin                : sun oct 27 2002
    copyright            : (C) 2002 by Piotr Mierzwiski
    email                : peterm@go2.pl

    copyright            : See COPYING file that comes with this project
***************************************************************************/

#ifndef _FILEVIEW_H_
#define _FILEVIEW_H_

#include <qtimer.h>
#include <qaccel.h>
#include <qaction.h>
#include <qlibrary.h>
#include <qmenubar.h>
#include <qtoolbar.h>
#include <qsplitter.h>
#include <qlistview.h>
#include <qtoolbutton.h>
#include <qstringlist.h>

#include "plugins/view/text/view.h"
#include "keyshortcuts.h"
#include "fileinfoext.h"
#include "statusbar.h"
#include "virtualfs.h" // for #include "filesassociation.h"

/**
 * Author:
 * Piotr Mierzwiski
 **/
/** @short Bazowa klasa obsugujca podgld pliku.
 Jej obiekt tworzy okno gwne podgldu, paskiem menu, narzdziowym, pasek
 statusu a take list boczn z list plikw wybranych przez uytkownika do
 podgldu. Specyficzne dla danego podgldu funkcje przedstawione w menu
 i na pasku narzdziowym, s pobierane z aktualnego obiektu podglgu pliku.
 Jej gwnym zadaniem jest adowanie pluginu wybranego i inicjowanie jego
 obiektu. Poredniczy rwnie w adowaniu pliku do pogldu, wykorzystujc
 do tego celu sygnaw (powinny by one poczone z obiektem systemu plikw
 z panelu, z ktrego uruchamianyjest podgld). Klasa zawiera obsug
 podstawowych funkcji z menu, gwnego, np. zmian rodzaju widoku - uruchamiajc
 odpowiedni procedur w obiekcie konkretnego podgldu. Zarzdza rwnie
 boczn (chowan) list plikw, adujc do niej nazwy i uruchamiajc podgld
 po wybraniu jednego z plikw.
*/
class FileView : public QWidget
{
	Q_OBJECT
public:
	/** Konstruktor klasy podgldu pliku.\n
	 @param filesList - lista plikw (ich nazwy s umieszczne na tzw.bocznej
	 licie),
	 @param currentItem - numer biecej nazwy pliku na licie @em filesList ,
	 @param editMode - czy podgld w trybie edycji,
	 @param parent - rodzic obiektu,
	 @param name - nazwa obieku.\n
	 Inicjowane s tutaj niezbdne skadowe klasy, inicjowany oglny widok, tworzone
	 obiekty wykorzystywane przez klas, inicjowane paski: menu i narzdziowy.
	 */
	FileView( const QStringList & filesList, uint currentItem, bool editMode=FALSE, QWidget *parent=NULL, const char *name=0 );

	/** Destruktor klasy.\n
	 Wyadowywany jest tutaj plugin ustawie podgldu oraz plugin biecego
	 podgldu, a take usuwane s wszystkie obiekty uywane wewntrz.
	 */
	~FileView();

	/** Metoda ustawia wskanik obiektu skojae plikw na podany.\n
	 @param fa - wskanik obiektu skojae plikw.
	 */
	void setFilesAssociation( FilesAssociation *fa ) { mFilesAssociation = fa; }

	/** Metoda ustawia wskanik obiektu skrtw klawiszowych na podany.\n
	 @param keyShortcuts - wskanik obiektu skrtw klawiszowych.
	 */
	void setKeyShortcuts( KeyShortcuts * keyShortcuts ) { mKeyShortcuts = keyShortcuts; }

	/** Zwraca ciek do aktualnie otwartego pliku.
	 @return absolutna cieka do aktualnie otwartego pliku.
	 */
	QString currentURL() const { return FileInfoExt::filePath(mCurrentFileName); }

	/**
	 Zwraca nazw aktualnie otwartego pliku.\n
	 @return nazwa z absolutn ciek do aktualnie otwartego pliku.
	 */
	QString fileName() const  { return mCurrentFileName; }

	/** Zwraca rodzaj biecego widoku.\n
	 @return rodzaj widoku, patrz te: @see KindOfView
	 */
	int kindOfView() const { return (int)mKindOfView; }

	/** Metoda suy do ustawienia rodzaju widoku dla biecego pliku na podany.\n
	 @param kindOfView - rodzaj widoku, patrz te: @see KindOfView.\n
	 Wywoywana jest tu tylko metoda @em createView().
	 */
	void setNewKindOfView( View::KindOfView kindOfView );

	/** Metoda powoduje wstawienie na tzw.boczn liste widoku podanej listy nazw.\n
	 @param filesList - lista nazwa plikw do wstawienia.\n
	 Metoda ta jest wywoywana z konstruktora.
	 */
	void insertFilesList( const QStringList & filesList );

private:
	View      *mView;
	QSplitter *mSplitter;
	QWidget   *mViewParent;
	StatusBar *mStatus;
	QListView *mSideListView;
	QLibrary  *mSettingsLib, *mViewLib;
	QWidget   *mSettingsWidget;

	FilesAssociation *mFilesAssociation;

	KindOfFile mKindOfFile;
	View::KindOfView mKindOfView;
	View::ModeOfView mModeOfView;

	QString mCurrentFileName, mInitFileName, mCurrentDirPath;

	bool m_bViewInWindow, m_bInitEditMode;
	QMenuBar    *mMenuBar;
	QPopupMenu  *mFileMenu, *mViewMenu, *mEditMenu, *mOptionsMenu;
	QPopupMenu  *mModeOfViewMenu, *mRecentFilesMenu;
	QAction     *mSaveA, *mSideListA;
	QToolBar    *mToolBar, *mAddonToolBar;

	QByteArray mBinBuffer;

	QTimer *mTimer;
	KeyShortcuts *mKeyShortcuts;
	QAction *mNewA, *mOpenA, *mReloadA, *mMenuBarA, *mToolBarA, *mConfigureA;
	QToolButton *mOpenBtn;

	uint mMB_Height, mTB_Height, mSB_Height;
	bool m_bShowMenuBar, m_bShowToolBar, m_bShowTheFilesList;
	uint mInitWidth, mInitHeigth, mSideListWidth;
	bool m_bItIsTextFile;

	QString mPluginsPath;
	QStringList mPluginsNameList;

	int mBytesRead;
	int mFileLoadMethod;
	bool m_bDetectIsBinOrTextFile;

	/** Identyfikatory akcji skrtw klawiszowych dla okna podgldu.
	*/
	enum ActionsOfView {
		Help_VIEW=0,
		Open_VIEW,
		Save_VIEW,
		SaveAs_VIEW,
		Reload_VIEW,
		OpenInNewWindow_VIEW,
		ShowHideMenuBar_VIEW,
		ShowHideToolBar_VIEW,
		ShowHideSideList_VIEW,
		ToggleToRawMode_VIEW,
		ToggleToEditMode_VIEW,
		ToggleToRenderMode_VIEW,
		ToggleToHexMode_VIEW,
		ShowSettings_VIEW,
		Quit_VIEW
	};


	/** Funkcja powoduje uaktualnienie paska menu oraz narzdziowego w zalenoci
	 od biecego rodzaju widoku oraz trybu podgldu.
	 */
	void updateMenuBar();

	/** Inicjowane jest tutaj menu oraz pasek narzdziowy.\n
	 @param parent - rodzic podgldu.\n
	 Pokazywany lub chowany jest take pasek menu oraz narzdziowy (w zalenoci
	 od informacji odczytanych w funkcji @em initView() ).
	*/
	void initMenuBarAndToolBar( QWidget * parent );

	/** Metoda powoduje utworzenie obiektu skrtw klawiszowych oraz
	 zainicjowanie jego listy domylnymi wartociami.
	 */
	void initKeyShortcuts();

	/** W funkcji inicjowany jest oglny widok podgldu.\n
	 Wczytywane i ustawiane s nastpujce waciwoci: rozmiar okna (jeli to
	 widok w oknie), informacje o pokazywniu paska menu oraz narzdziowego.
	 Nastpuje tu tak wykrycie cieki do pluginw (obsuga konkretnych
	 podgldw).
	*/
	void initView();

	/** Metoda powoduje usunicie obiektu plugina-podgldu oraz wyadowanie
	 plugina.
	 */
	void removeView();

protected:
	/** Metoda tworzy pusty widok podanego rodzaju, jli rodzaj zostanie
	 pominity, wtedy jest on odgadywany na podstawie rozszerzenia podanego pliku.\n
	 @param fileName - nazwa pliku wykorzystywana przy wykrywaniu rodzaju widoku;
	 @param kindOfView - rodzaj widoku do utworzenia (podanie UNKNOWNview, wymusza
	 jego detekcj na podstawie rozszerzenia podanego pliku);
	 @return - TRUE, jeli widok zosta pomylnie utworzony, w przeciwnym razie
	 FALSE.\n
	 Aby utworzy nowy widok adowany i uruchamiany jest okrelony plugin widoku.
	 Inicjowana jest tutaj take, skadowa @em mKindOfFile.
	 */
	bool createView( const QString & fileName, View::KindOfView kindOfView=View::UNKNOWNview );

	/** Metoda powoduje uruchomienie tzw. adowania pliku z opnieniem.\n
	 @param fileName - nazwa pliku do zaadowania.\n
	 W metodzie inicjowana jest skadowa @em mInitFileName oraz wywoywany slot:
	 @em slotReadyRead() .
	 */
	void loadingLate( const QString & fileName );


	/** Przechwytywane s tutaj wszelkie wcinicia klawiszy.\n
	 Funkcja zawiera tylko obsug klawisza Escape, ktry suy do zamknicia
	 okna.
	 */
	void keyPressEvent( QKeyEvent * );

	/** Funkcja wywoywana w momencie zmiany rozmiaru podgldu.\n
	 Uaktualniane s tutaj pozycje i rozmiary obiektw okna podgldu.
	*/
	void resizeEvent( QResizeEvent * );

	/** Funkcja wywoywana w momencie zamknicia podgldu.\n
	 Wykorzystywana jest do sprawdzania czy zawarto widoku zostaa zmieniona,
	 jeli tak, wtedy nastpuje zapytanie urzytkownika o zapis do pliku.
	 */
	void closeEvent( QCloseEvent * );

	QString kindOfViewStr( View::KindOfView ); // FUN.TYMCZASOWA - TYLKO DO TESTOW

public slots:
	/** W slocie inicjowane jest wczytywanie pliku.\n
	 Nastpuje tu pokazanie okna (jeli to widok w oknie) oraz pokazanie lub
	 schowanie bocznej listy. Jeli wczytywany plik ley w lokalnym systemie
	 plikw, wtedy wywoywany jest asynchroniczne slot @em slotLoadFile() ,
	 w przeciwnym razie wywoywany jest slot @em slotLoadFile() (tutaj
	 wczytywaniem zajmuje si obiekt klasy FileSystemu).
	 */
	void slotReadyRead();

private slots:
	/** Slot przygotowywuje podgld do wczytania pierwszego fragmentu (lub
	 caoci) pliku.\n
	 Nazwa wczytywanego pliku musi si znajdowa w skadowej @em mInitFileName ,
	 poniewa inicjuje ona z koleji skadow @em mCurrentFileName . Ustawiany jest
	 tutaj maksymalny rozmiar wczytywanego na raz bloku. Nastpnie nastpuje
	 wywoanie asynchroniczne slotu @em slotReadFile() . Na kocu inicjowana
	 jest boczna lista plikw.
	 */
	void slotLoadFile();

	/** W slocie nastpuje (asynchroniczne) wczytywanie (partiami w caoci) pliku
	 oraz uaktualnianie biecego obiektu podgldu.\n
	 Po wczytaniu pierwszego fragmentu lub caoci (w zalenoci od ustawionej
	 wczeniej metody wczytywania pliku) wykrywane jest, czy to plik tekstowy czy
	 binarny, a nastpnie tworzony waciwy obiekt podgldu, po czym uaktualniane
	 menu i pasek narzdziowy. Na kocu uaktualniany jest obiekt podgldu.\n
	 adowanie pliku nastpuje w sposb asynchroniczny, jeli plik ley w lokalnym
	 systemie plikw. Wczytywaniem zajmuj si sloty w odpowiednich systemach
	 plikw, komunikacja wykonywana jest za pomoc sygnau @em signalReadFile() ,
	 ktrego parametrami s: nazwa wczytywanego pliku, adres bufora, rozmiar
	 wczytywanego (na raz) bloku bajtw oraz metoda adowania -
	 @em mFileLoadMethod .
	 */
	void slotReadFile();

	/** Slot powoduje ustawienie podanego trybu podgldu dla biecego widoku.\n
	 @param itemId - numer trybu podgldu (w menu), musi by ona zgodna
	 z typem @em View::ModeOfView .
	 */
	void slotChangeModeOfView( int itemId );

	/** Slow wywoywany w momencie wybrananie przez uytkownika pliku z menu
	 ostatnio zaadowanych plikw.\n
	 @param itemId - numer nazwy pliku w menu.\n
	 Podejmuje prb wczytania pliku o wybranej nazwie.
	 */
	void slotItemOfRecentFilesMenuActivated( int itemId );


	/** Slot powoduje pokazanie dialogu adowania pliku.\n
	 Wybrany plik jest wczytywany do nowo utworzonego podgldu.
	 */
	void slotNew();

	/** Slot powoduje pokazanie dialogu adowania pliku.\n
	 Wybrany plik jest wczytywany do biecego podgldu.
	 */
	void slotOpen();

	/** Slot zapisuje podgld do biecego pliku, tylko jeli zosta zmieniony.
	 */
	void slotSave();

	/** Slot pokazuje dialog zapisu pliku, w celu zapisania biecego pod podan
	 tam nazw.
	 */
	void slotSaveAs();

	/** Slot powoduje ponowne zaadowanie aktualnie wczytanego pliku.
	 */
	void slotReload();


	/** Slot powoduje pokazanie lub schowanie paska menu.\n
	 Przy czym, osadzony w panelu podgld ma zawsze ukrywane menu.
	 */
	void slotShowHideMenuBar();

	/** Slot powoduje pokazanie lub schowanie paska narzdziowego.
	 */
	void slotShowHideToolBar();

	/** Slot powoduje pokazanie ustawie dla podgldu.\n
	 adowany jest w tym celu plugin. Jeli operacja zakoczy si sukcesem, wtedy
	 czone s niezbdne sygnay ze slotami.
	 */
	void slotShowSettings();

	/** W slocie wczytywane s z pliku konfiguracyjnego nazwy plikw pokazywane na
	 bocznej licie.\n
	 Po wczytaniu nastpuje wstawienie ich na boczn list.
	 */
	void slotLoadFilesList();

	/** W slocie zapisywana jest do pliku konfiguracyjnego lista nazw plikw
	 z bocznej listy.
	 */
	void slotSaveFilesList();

	/** Slot powoduje pokazanie lub ukrycie (w zalenoci od wartoci skadowej
	 @em mShowTheFilesList) bocznej listy plikw.
	 */
	void slotShowTheFilesList();

	/** Slot powoduje zaadowanie kliknitego na bocznej licie pliku.\n
	 @param item - wskanik elementu bocznej listy.
	 */
	void slotOpenCurrentItem( QListViewItem * item );

	/** Slot zaimplementowany tylko dla wykonania poprawnego czenie sygnau ze
	 slotem.\n Wewntrz wywoywany jest tylko slot @em slotOpenCurrentItem() .
	 */
	void slotOpenCurrentItem( QListViewItem * item, const QPoint & , int );

	/** Slot powoduje zastosowanie ustawie skrtw klawiszowych dla gwnego
	 okna podgldu (biecej klasy).\n
	 @param keyShortcuts - wskanik na obiekt skrtw klawiszowych.
	 */
	void slotApplyKeyShortcuts( KeyShortcuts * keyShortcuts );

signals:
	/** Sygna uywany do asynchronicznego wczytywania pliku.\n
	 @param fileName - nazwa wczytywanego pliku,
	 @param buffer - adres bufora, w ktrym naley umieszcza wczytywane dane,
	 @param bytesRead - zmienna, w ktrej zapisywana bdzie ilo wczytanych
	 bajtw, zawiera informacj o maksymalnej iloci bajtw do wczytania.
	 @param fileLoadMethod - metoda wczytywania pliku, patrz te:
	  @see View::UpdateMode.\n
	 Sygna powinien by poczony ze slotem w obiekcie klasy okrelonego systemu
	 plikw.
	 */
	void signalReadFile( const QString & fileName, QByteArray & buffer, int & bytesRead, int fileLoadMethod );

	/** Sygna suy do ustawienia informacji o biecym widoku w obiekcie
	 ustawie, (jest on skadow tej klasy). W obiekcie ustawie informacja ta
	 jest uywana przy zastosowywaniu ustawie dla biecego widoku.\n
	 @param currentKindOfView - rodzaj widoku, patrz te: @see View::KindOfView
	 */
	void signalSetCurrentKindOfView( View::KindOfView currentKindOfView );

	/** Sygna suy do ustawienia informacji o skrtach klawiszowych w obiekcie
	 ustawie, (jest on skadow tej klasy).\n
	 @param keyShortcuts - wskanik na obiekt skrtw klawiszowych.
	 */
	void signalSetKeyShortcuts( KeyShortcuts * keyShortcuts );

	/** Sygna wysyany po operacji zapisu pliku i suy do poinformowania panela
	 o potrzebie uaktualniania informacji o tym pliku na licie.\n
	 @param fileName - nazwa pliku, ktry zosta zapisany
	 */
	void signalUpdateItemOnLV( const QString & fileName );

	/** Sygna wysyany w momencie wcinicia guzika zamknicia widoku, ktry
	znajduje si na pasku narzdziowy.\n
	Guzik zamykania pokazywany jest tylk jeli widok jest osadzony w panelu.
	 */
	void signalClose();

	/** Sygna wysyany w momencie zapisu ustawie kadego typu podgldu.
	 */
	void signalRereadFilesAssociation();

};

#endif
