/***************************************************************************
                          squeezelabel.h  -  description
                             -------------------
    begin                : sun aug 10 2003
    copyright            : (C) 2003 by Piotr Mierzwiski
    email                : peterm@go2.pl

    Copyright: See COPYING file that comes with this project
***************************************************************************/

#ifndef _SQUEEZELABEL_H
#define _SQUEEZELABEL_H

#include <qlabel.h>

/**
 * Author:
 * Piotr Mierzwiski
 **/
/**
 @short Klasa etykiety tekstowej z moliwoci jej ciskania.
 Klasa spenia zadanie takie same jak QLabel. Jedyn jej przewag jest
 moliwo tzw. ciskania tekstu w szerokoci jak ma aktualnie obiekt tej
 klasy. Jeeli skalowanie zawartoci jest wczone (domylnie), wtedy jest
 sprawdzane czy cig mieci si w caoci na etykiecie, jeli nie wtedy
 jest on skracany. Polega to na tym, e pokazywany jest fragment pocztkowy
 i kocowy, natomiast rodek jest zastpowany przez cig "...". Moliwe jest
 pokazywanie caoci tekstu po najechaniu nad nim kursorem myszy (waciwo
 ta jest domylnie wczona).
*/
class SqueezeLabel : public QLabel
{
	Q_OBJECT
public:
	/** Konstruktor klasy.\n
	 @param parent - wskanik na rodzica dla obiektu,
	 @param name - nazwa dla obiektu,
	 @param f - flaga typu Qt::WFlags (patrz dokumentacja dot. klasy Qt:QLabel).\n
	 Wczane jest tutaj pokazywanie dymka z caoci cinitego tekstu oraz
	 skalowanie zawartoci i inicjowane s skadowe klasy.
	*/
	SqueezeLabel( QWidget * parent, const char * name=0, WFlags f=0 );

	/** Destruktor.\n
	 Brak definicji (jest pusta).
	 */
	virtual ~SqueezeLabel() {}

	/** Metoda powoduje wczenie lub wyczenie pokazywania dymku z penym
	 tekstem skrconego napisu.\n
	 @param show - warto TRUE wcza pokazywanie dymku, FALSE - wycza.\n
	 Waciwoc ta jest domylnie wczona.
	 */
	void setShowToolTip( bool show );

private:
	bool mShowTip;
	QString mFullText;


	/** Funkcja wykorzystywana do ustawiania kolejnych 'cinitych' linii tekstu
	 oraz dodajca tzw. dymek (jeli waciwo ta zostaa wczona) do obiektu.\n
	 @param maxWidth - maksymalna szeroko obiektu (w punktach obrazu).
	 */
	void setSqueezingText( uint maxWidth );

	/** Waciwa funkcja skracajca podan linie tekstu do podanej szerokoci.\n
	 @param line - linia tekstu do skrcenia,
	 @param maxWidth - maksymalna szeroko obiektu (w punktach obrazu).
	 */
	QString squeezeLine( const QString &, uint );

public slots:
	/** Slot sucy do ustawiania tekstu na obiekcie.\n
	 @param text - tekst do ustawiania.
	 */
	virtual void setText( const QString & text );

protected:
	/** Obsuga zdarzenie zmiany rozmiaru.\n
	 Uruchamiana jest tu procedura dopasowania tekstu na obiekcie do jego
	 szerokoci, przy kadorazowym zmianie rozmiaru obiektu.
	 */
	virtual void resizeEvent( QResizeEvent * );

};

#endif
