/***************************************************************************
                          listviewext.h  -  description
                             -------------------
    begin                : Mon Jul 9 2001
    copyright            : (C) 2001 by Mariusz Borowski (initializer)
                         : (C) 2002,2003 by Piotr Mierzwiski (developer)
    emails               : mborowski@silesia.net, peterm@go2.pl

    copyright            : See COPYING file that comes with this project
 ***************************************************************************/

#ifndef LISTVIEWITEMEXT_H
#define LISTVIEWITEMEXT_H

#include "urlinfoext.h"

#include <qlistview.h>
#include <qtooltip.h>

/**
  *@authors: Mariusz Borowski (init), Piotr Mierzwiski (develop)
  */
class ListViewExt;

/**
 @short Klasa dymku dla elementu listy.
 Jest ona wykorzystywana do pokazywania penych nazw przycitych komrek listy
 oraz dla pokazywania cieek docelowych dla linkw.
 */
class ListViewExtToolTip : public QToolTip
{
public:
	/** Konstruktor klasy.\n
	 @param parent - wskanik na rodzica,
	 @param lv - wskanik na list, dla ktrej elementw bdzie pokazywany dymek.\n
	 Ustawiany jest tutaj rozmiar fontu dla dymku.
	 */
	ListViewExtToolTip( QWidget * parent, ListViewExt *lv );

	/** Metoda wywoywana jest tylko wtedy, gdy kursor znajdzie si na elementem.\n
	 @param pos - pozycja kursora.\n
	 Jest tu pokazywana informacja w postaci dymku, dla przycitego tekstu w
	 komrce nad ktr kursor si aktualnie znajduj. Jeli plik pod kursorem
	 jest linkiem wtedy pokazywana jest jego cieka docelowa (element listy
	 trzyma tak informacj).
	 */
	void maybeTip( const QPoint & pos );

private:
	ListViewExt *mView;

};


/** @short Klasa elementu widoku listy.
 Klasa zawiera metody pomagajce w uywaniu jej jako obiektu elementu listy
 plikw. Oferuje nastpujce waciwoci:
  @li zwraca informacj czy biecy element jest katalogiem czy plikiem;
  @li zwraca absolutn ciek dla biecego elementu (jeli widokiem jest lista
  wtedy wstawiany jest bieca cieka);
  @li wcza/wycza pokazywanie rozmiarw katalogw
  @li umoliwia ukrycie i pokazanie elementu
  @li zwraca informacj typu UrlInfoExt (atrybuty pliku).\n
 Obiekty tej klasy mona inicjowa tylko obiektami klasy @em UrlInfoExt poprzez
 jej konstruktory oraz za pomoc metod. Klasa zawiera dwie metody rysowania
 kursora na licie, mianowicie: jako dwukolorowa ramka oraz wypeniona jednym
 kolorem belka.
*/
class ListViewItemExt : public QListViewItem
{
public:
	/** Konstruktor klasy.\n
	 Inicjowane s tu skadowe klasy.
	 */
	ListViewItemExt( ListViewExt *parent );

	/** Konstruktor klasy.\n
	 Inicjowane s tu skadowe klasy. Konstruktor uywany jest przez widok listy,
	 widok drzewa uywa go tylko do wstawienia elementu korzenia.
	 */
	ListViewItemExt( ListViewExt *parent, const UrlInfoExt & urlInfo );

	/** Konstruktor klasy.\n
	 Inicjowana jest tylko klasa, po ktrej dziedziczy bieca.
	 */
	ListViewItemExt( ListViewItemExt *parent );

	/** Konstruktor klasy.\n
	 Inicjowane s tu skadowe klasy. Konstruktor uywany jest przez widok drzewa.
	 */
	ListViewItemExt( ListViewItemExt *parent, const UrlInfoExt & urlInfo );

	/** Przeciona metoda klasy QListViewItem, ktra jest odpowiedzialna
	 ze rysowanie komrki elementu listy oraz ta dla kadej z nich.\n
	 @param p - wskanik 'rysownika' zwizany z otwartym urzdzeniem rysujcym \n
	 @param cg - grupa kolorw do uycia \n
	 @param col - bieca kolumna w ktrej nastpuje rysowanie \n
	 @param width - szeroko biecej kolumny \n
	 @param alignment - strona do ustawiania zawartoci rysowanego elementu (lewa,centrum,prawa)
	 */
	virtual void paintCell(  QPainter *p, const QColorGroup & cg, int col, int widht, int alignment );

	/** Funkcja odpowiedzialna za rysowanie kursora wraz z zawartoci.
	 */
	virtual void paintFocus( QPainter *p, const QColorGroup & cg, const QRect & r );

	/** Funkcja wykonuje korekcj sortowania.\n
	 Poprawka polega na umieszczaniu katalogw na pocztku, a elementu ".." (jeli
	 jest) na samej grze. Korygowane jest te sortowanie po rozmiarze.
	 */
	virtual QString key( int column, bool ascending ) const;

	/** Zwraca zawarto podanej komrki elementu.
	 @param column - kolumna z ktrej naley zwrci warto.\n
	 Przy czy w przypadku kolumny rozmiarw zaley to od typu elementu.
	 Jeli biecym elementem jest katalog, wtedy zwracana jest, w zalenoci od
	 wartoci skadowej @em mShowDirSize , dla wartoci TRUE - rozmiar katalogu
	 (w formacie @em BKBMBGBformat ), dla wartoci FALSE - cig "\<DIR\>". Jeeli
	 elementem jest plik (lub link), wtedy zwracany jest rozmiar (niesformatowany)
	 */
	virtual QString text( int column ) const;

	/** Ustawia w podanym numerze komrki odpowiadajcej jej podan warto.\n
	 @param column - numer komrki, w ktrym naley zmieni warto,
	 @param str - nowa warto dla podanej komrki.
	 */
	virtual void setText( int column, const QString & str );

	/** Zwraca informacj o widoczno elementu.\n
	 @return TRUE jeli element jest widoczny, w przeciwnym razie FALSE.
	 */
	virtual bool isVisible() const { return height(); }

	/** Pokazuje drzewo lub zwyk list plikw, w zalenoci od rodzaju widoku
	 ustawionego w klasie macieystej.
	 @param open - jeli rwne TRUE, wtedy plus przy elemencie zmieniany jest na
	 minus, dla FALSE dzieje si odwrotnie. Dodatkowo w przypadku, gdy elementem
	 jest archiwum zmieniana jest ikona na otwart (TRUE) lub zamknit (FALSE)
	 paczk.\n
	 Uywane przy widoku drzewa.\n
	 */
	void setOpen( bool open );

	/** Metoda zwraca nazw biecego pliku wraz z absolutn ciek do niego,
	 przyczym nazwa nie zawiera nazwy hosta ('/', 'ftp://', itp.).
	 @param addSlashIfDir jest rwny TRUE i elementem jest katalog, wtedy do nazwy
	 dodawany jest znak '/'.
	 */
	QString fullName( bool addSlashIfDir=TRUE );

	/** Metoda zwraca obiekt klasy @em UrlInfoExt z informacj o pliku.
	 @return obiekt klasy @em UrlInfoExt .
	*/
	UrlInfoExt entryInfo() const { return mEntryInfo; }

	/** Zwraca informacj o tym, czy biecy element jest katalogiem.
	 @return TRUE jeli element jest katalogiem, w przeciwnym razie FALSE.
	 Przy czym kady link jest zawsze traktowany jak plik.
	 */
	bool isDir() const { return mIsDir; }

	/** Wcza pokazywanie rozmiarw katalogw zamiast cigu "\<DIR\>".
	 */
	void setShowDirSize( bool show ) { mShowDirSize = show; }

	/** Metoda powoduje ukrycie elementu.
	 */
	void hide();

	/** Metoda powoduje pokazanie elementu.
	 */
	void show();

private:
	QHeader         *mHeader;
	ListViewExt     *mListView;
	ListViewItemExt *mListViewItemExt;

	QString mTxt, ext;
	UrlInfoExt mEntryInfo;
	bool mIsDir, mShowDirSize;

	typedef QValueList<UrlInfoExt> UrlInfoList;
	UrlInfoList::iterator it;


	/** Zwraca wskanik do listy rodzica.\n
	 @return wskanik do listy rodzica.
	 */
	ListViewExt *listViewExt() const { return (ListViewExt *)QListViewItem::listView(); }

	/** Ustawia podan pixmap w pierwszej komrce elementu.\n
	 @param p - pixmapa do ustawienia.
	 */
	void setPixmap( QPixmap p );

	/** Ustawia odpowiedni waciwo 'expandable' dla elementu - pliku.\n
	 Uywane przy widoku drzewa.\n Jeli element jest katalogiem, a biecy
	 uytkownik moe go czyta i nie jest on pusty, wtedy waciwo 'expandable'
	 jest ustawiana na TRUE, w przeciwnym razie oraz w przypadku pliku nastpuje
	 ustawienie na FALSE.
	 */
	void setup();

	/** Zwraca przycity, podany cig tekstowy, ktry jest dopasowany do
	 szerokoci podanego numeru kolumny listy.\n
	 @param str - cig tekstowy do przycicia,
	 @param column - numer kolumny, w ktrej znajduje si podany cig,
	 @param columnWidth - szeroko kolumny, w ktrej znajduje si podany cig,
	 @param fontWeight - TRUE jeli cig jest pisany fontem QFont::bold,
	 w przeciwnym razie FALSE.
	 */
	QString trimedString( const QString & str, const int & column, const int & columnWidth, const int & fontWeight );

};

#endif
